@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2005, 2007-2012, 2014-2016 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,12 +13,11 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "comsat.h"
-#include "mailutils/libargp.h"
+#include "mailutils/syslog.h"
+#include "mailutils/cli.h"
 
 #ifndef PATH_DEV
 # define PATH_DEV "/dev"
@@ -36,10 +35,8 @@ extern void setutent (void);
 extern struct utmp *getutent (void);
 #endif
 
-#ifdef UTMPX
-# ifdef HAVE_UTMPX_H
-#  include <utmpx.h>
-# endif
+#ifdef HAVE_UTMPX_H
+# include <utmpx.h>
 typedef struct utmpx UTMP;
 # define SETUTENT() setutxent()
 # define GETUTENT() getutxent()
@@ -49,64 +46,84 @@ typedef struct utmp UTMP;
 # define SETUTENT() setutent()
 # define GETUTENT() getutent()
 # define ENDUTENT() endutent()
+# if !HAVE_STRUCT_UTMP_UT_USER
+#  if HAVE_STRUCT_UTMP_UT_NAME
+#   define ut_user ut_name
+#  else
+#   error "Neither ut_user nor ut_name found in struct utmp. Please report."
+#  endif
+# endif
 #endif
 
 #define MAX_TTY_SIZE (sizeof (PATH_TTY_PFX) + sizeof (((UTMP*)0)->ut_line))
 
 const char *program_version = "comsatd (" PACKAGE_STRING ")";
-static char doc[] = "GNU comsatd";
-static char args_doc[] = N_("\n--test MBOX-URL MSG-QID");
 
-#define OPT_FOREGROUND 256
+int test_mode;
+char *biffrc = BIFF_RC;
+mu_m_server_t server;
 
-static struct argp_option options[] = 
+static void
+set_inetd_mode (struct mu_parseopt *po, struct mu_option *opt,
+		char const *arg)
 {
-  { "config", 'c', N_("FILE"), OPTION_HIDDEN, "", 0 },
-  { "convert-config", 'C', N_("FILE"), 0,
-    N_("convert the configuration FILE to new format"), 0 },
-  { "test", 't', NULL, 0, N_("run in test mode"), 0 },
-  { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), 0},
-  { "inetd",  'i', 0, 0, N_("run in inetd mode"), 0 },
-  { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("runs in daemon mode with a maximum of NUMBER children"), 0 },
-  { NULL, 0, NULL, 0, NULL, 0 }
-};
+  mu_m_server_set_mode (server, MODE_INTERACTIVE);
+}
+  
+static void
+set_daemon_mode (struct mu_parseopt *po, struct mu_option *opt,
+		 char const *arg)
+{
+  mu_m_server_set_mode (server, MODE_DAEMON);
+  if (arg)
+    {
+      size_t max_children;
+      char *errmsg;
+      int rc = mu_str_to_c (arg, mu_c_size, &max_children, &errmsg);
+      if (rc)
+	{
+	  mu_parseopt_error (po, _("%s: bad argument"), arg);
+	  exit (po->po_exit_error);
+	}
+      mu_m_server_set_max_children (server, max_children);
+    }
+}
 
-static error_t comsatd_parse_opt (int key, char *arg,
-				  struct argp_state *state);
+static void
+set_foreground (struct mu_parseopt *po, struct mu_option *opt,
+		char const *arg)
+{
+  mu_m_server_set_foreground (server, 1);
+}
 
-static struct argp argp = {
-  options,
-  comsatd_parse_opt,
-  args_doc, 
-  doc,
-  NULL,
-  NULL, NULL
-};
-
-static const char *comsat_argp_capa[] = {
-  "common",
-  "debug",
-  "logging",
-  "mailbox",
-  "locking",
-  "license",
-  NULL
-};
+static struct mu_option comsat_options[] = {
+  { "test", 't', NULL, MU_OPTION_DEFAULT,
+    N_("run in test mode"),
+    mu_c_bool, &test_mode },
+  { "foreground",  0, NULL, MU_OPTION_DEFAULT,
+    N_("remain in foreground"),
+    mu_c_bool, NULL, set_foreground },
+  { "inetd",  'i', NULL, MU_OPTION_DEFAULT,
+    N_("run in inetd mode"),
+    mu_c_bool, NULL, set_inetd_mode },
+  { "daemon", 'd', N_("NUMBER"), MU_OPTION_ARG_OPTIONAL,
+    N_("runs in daemon mode with a maximum of NUMBER children"),
+    mu_c_string, NULL, set_daemon_mode },
+  { "file", 'f', N_("FILE"), MU_OPTION_DEFAULT,
+    N_("read FILE instead of .biffrc"),
+    mu_c_string, &biffrc },
+  MU_OPTION_END
+}, *options[] = { comsat_options, NULL };
 
 #define SUCCESS 0
 #define NOT_HERE 1
 #define PERMISSION_DENIED 2
 
-#ifndef MAXHOSTNAMELEN
-# define MAXHOSTNAMELEN 64
-#endif
-
 int maxlines = 5;
-char hostname[MAXHOSTNAMELEN];
+char *hostname;
 const char *username;
 int require_tty;
-mu_m_server_t server;
+int biffrc_errors = BIFFRC_ERRORS_TO_TTY | BIFFRC_ERRORS_TO_ERR;
 
 static void comsat_init (void);
 static int comsat_main (int fd);
@@ -117,96 +134,80 @@ static char *mailbox_path (const char *user);
 static int change_user (const char *user);
 
 static int reload = 0;
-int test_mode;
+static int
+biffrc_error_ctl (mu_config_value_t *val, int flag)
+{
+  int res;
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  if (mu_str_to_c (val->v.string, mu_c_bool, &res, NULL))
+    mu_diag_output (MU_LOG_ERROR, _("not a boolean"));
+  else if (res)
+    biffrc_errors |= flag;
+  else
+    biffrc_errors &= ~flag;
+  return 0;
+}
+    
+static int
+cb_biffrc_errors_to_tty (void *data, mu_config_value_t *val)
+{
+  return biffrc_error_ctl (val, BIFFRC_ERRORS_TO_TTY);
+}
+
+static int
+cb_biffrc_errors_to_err (void *data, mu_config_value_t *val)
+{
+  return biffrc_error_ctl (val, BIFFRC_ERRORS_TO_ERR);
+}
 
 struct mu_cfg_param comsat_cfg_param[] = {
-  { "allow-biffrc", mu_cfg_bool, &allow_biffrc, 0, NULL,
+  { "allow-biffrc", mu_c_bool, &allow_biffrc, 0, NULL,
     N_("Read .biffrc file from the user home directory.") },
-  { "require-tty", mu_cfg_bool, &require_tty, 0, NULL,
+  { "require-tty", mu_c_bool, &require_tty, 0, NULL,
     N_("Notify only if the user is logged on one of the ttys.") },
-  { "max-lines", mu_cfg_int, &maxlines, 0, NULL,
+  { "biffrc-errors-to-tty", mu_cfg_callback, NULL, 0, cb_biffrc_errors_to_tty,
+    N_("Send biffrc errors to user's tty."),
+    N_("arg: bool") },
+  { "biffrc-errors-to-err", mu_cfg_callback, NULL, 0, cb_biffrc_errors_to_err,
+    N_("Send biffrc errors to Mailutils error output."),
+    N_("arg: bool") },
+  { "max-lines", mu_c_int, &maxlines, 0, NULL,
     N_("Maximum number of message body lines to be output.") },
-  { "max-requests", mu_cfg_uint, &maxrequests, 0, NULL,
+  { "max-requests", mu_c_uint, &maxrequests, 0, NULL,
     N_("Maximum number of incoming requests per request control interval.") },
-  { "request-control-interval", mu_cfg_time, &request_control_interval,
+  { "request-control-interval", mu_c_time, &request_control_interval,
     0, NULL,
     N_("Set control interval.") },
-  { "overflow-control-interval", mu_cfg_time, &overflow_control_interval,
+  { "overflow-control-interval", mu_c_time, &overflow_control_interval,
     0, NULL,
     N_("Set overflow control interval.") },
-  { "overflow-delay-time", mu_cfg_time, &overflow_delay_time,
-    0, NULL,
+  { "overflow-delay-time", mu_c_time, &overflow_delay_time,
+   0, NULL,
     N_("Time to sleep after the first overflow occurs.") },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("Server configuration.") },
   { NULL }
 };
 
-static error_t
-comsatd_parse_opt (int key, char *arg, struct argp_state *state)
-{
-  static struct mu_argp_node_list lst;
+static char const *alt_args[] = { N_("--test MBOX-URL MSG-QID"), NULL };
 
-  switch (key)
-    {
-    case 'c':
-      {
-	char *cfg;
-	int fd;
-	FILE *fp;
+static struct mu_cli_setup cli = {
+  .optv = options,
+  .cfg =comsat_cfg_param,
+  .prog_doc = N_("GNU comsatd -- notify users about incoming mail"),
+  .prog_alt_args = alt_args,
+  .server = 1
+};
 
-	mu_diag_output (MU_DIAG_WARNING,
-_("The old configuration file format and the --config command\n"
-  "line option are deprecated and will be removed in the future\n"
-  "release. Please use --convert-config option to convert your\n"
-  "settings to the new format."));
-	/* FIXME: Refer to the docs */
-	
-	fd = mu_tempfile (NULL, &cfg);
-	fp = fdopen (fd, "w");
-	convert_config (arg, fp);
-	fclose (fp);
-	mu_get_config (cfg, mu_program_name, comsat_cfg_param, 0, NULL);
-	unlink (cfg);
-	free (cfg);
-      }
-      break;
-      
-    case 'C':
-      convert_config (arg, stdout);
-      exit (0);
-
-    case 'd':
-      mu_argp_node_list_new (&lst, "mode", "daemon");
-      if (arg)
-	mu_argp_node_list_new (&lst, "max-children", arg);
-      break;
-
-    case 'i':
-      mu_argp_node_list_new (&lst, "mode", "inetd");
-      break;
-
-    case OPT_FOREGROUND:
-      mu_argp_node_list_new (&lst, "foreground", "yes");
-      break;
-
-    case 't':
-      test_mode = 1;
-      break;
-      
-    case ARGP_KEY_INIT:
-      mu_argp_node_list_init (&lst);
-      break;
-      
-    case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (&lst, NULL, NULL);
-      break;
-
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
-}
+static char *capa[] = {
+  "debug",
+  "logging",
+  "mailbox",
+  "locking",
+  NULL
+};
 
 static RETSIGTYPE
 sig_hup (int sig)
@@ -218,11 +219,17 @@ sig_hup (int sig)
 void
 comsat_init ()
 {
+  int rc;
+
   /* Register mailbox formats */
   mu_register_all_mbox_formats ();
 
-  gethostname (hostname, sizeof hostname);
-
+  rc = mu_get_host_name (&hostname);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_get_host_name", NULL, rc);
+      exit (EXIT_FAILURE);
+    } 
   /* Set signal handlers */
   signal (SIGTTOU, SIG_IGN);
   signal (SIGCHLD, SIG_IGN);
@@ -265,7 +272,7 @@ comsat_process (char *buffer, size_t rdlen)
     {
       if (require_tty)
 	return;
-      strcpy (tty, "/dev/null");
+      tty[0] = 0;
     }
 
   /* Child: do actual I/O */
@@ -276,7 +283,7 @@ int
 comsat_main (int fd)
 {
   int rdlen;
-  int len;
+  socklen_t len;
   struct sockaddr fromaddr;
   char buffer[216]; /*FIXME: Arbitrary size */
 
@@ -307,7 +314,8 @@ static unsigned overflow_count = 0; /* Number of overflows detected during
 				       the current interval */
 
 int
-comsat_prefork (int fd, void *data, struct sockaddr *s, int size)
+comsat_prefork (int fd, struct sockaddr *s, int size,
+		struct mu_srv_config *pconf, void *data)
 {
   int retval = 0;
   time_t now;
@@ -354,28 +362,28 @@ comsat_prefork (int fd, void *data, struct sockaddr *s, int size)
 
 int
 comsat_connection (int fd, struct sockaddr *sa, int salen,
-		   void *data, mu_ip_server_t srv,
-		   time_t to, int transcript)
+		   struct mu_srv_config *pconf, void *data)
 {
   char *buffer;
   size_t rdlen, size;
 
-  if (mu_udp_server_get_rdata (srv, &buffer, &rdlen))
+  if (mu_udp_server_get_rdata (pconf->tcpsrv, &buffer, &rdlen))
     return 0;
-  if (transcript)
+  if (pconf->transcript)
     {
       char *p = mu_sockaddr_to_astr (sa, salen);
       mu_diag_output (MU_DIAG_INFO,
-		      ngettext ("received %d byte from %s",
-				"received %d bytes from %s", rdlen),
-		      rdlen, p);
+		      ngettext ("received %lu byte from %s",
+				"received %lu bytes from %s",
+				(unsigned long) rdlen),
+		      (unsigned long) rdlen, p);
       mu_diag_output (MU_DIAG_INFO, "string: %s", buffer);
       free (p);
     }
-  mu_udp_server_get_bufsize (srv, &size);
+  mu_udp_server_get_bufsize (pconf->tcpsrv, &size);
   if (size < rdlen + 1)
     {
-      int rc = mu_udp_server_set_bufsize (srv, rdlen + 1);
+      int rc = mu_udp_server_set_bufsize (pconf->tcpsrv, rdlen + 1);
       if (rc)
 	{
 	  mu_error (_("cannot resize buffer: %s"), mu_strerror (rc));
@@ -387,43 +395,18 @@ comsat_connection (int fd, struct sockaddr *sa, int salen,
   return 0;
 }
 
-static const char *
-get_newline_str (FILE *fp)
-{
-#if defined(OPOST) && defined(ONLCR)
-  struct termios tbuf;
-
-  tcgetattr (fileno (fp), &tbuf);
-  if ((tbuf.c_oflag & OPOST) && (tbuf.c_oflag & ONLCR))
-    return "\n";
-  else
-    return "\r\n";
-#else
-  return "\r\n"; /* Just in case */
-#endif
-}
-
 /* NOTE: Do not bother to free allocated memory, as the program exits
    immediately after executing this */
 static void
 notify_user (const char *user, const char *device, const char *path,
 	     mu_message_qid_t qid)
 {
-  FILE *fp;
-  const char *cr;
   mu_mailbox_t mbox = NULL;
   mu_message_t msg;
   int status;
 
   if (change_user (user))
     return;
-  if ((fp = fopen (device, "w")) == NULL)
-    {
-      mu_error (_("cannot open device %s: %s"), device, mu_strerror (errno));
-      return;
-    }
-
-  cr = get_newline_str (fp);
 
   if (!path)
     {
@@ -432,8 +415,9 @@ notify_user (const char *user, const char *device, const char *path,
 	return;
     }
 
-  if ((status = mu_mailbox_create (&mbox, path)) != 0
-      || (status = mu_mailbox_open (mbox, MU_STREAM_READ|MU_STREAM_QACCESS)) != 0)
+  if ((status = mu_mailbox_create (&mbox, path)) != 0 ||
+      (status = mu_mailbox_open (mbox,
+				 MU_STREAM_READ|MU_STREAM_QACCESS)) != 0)
     {
       mu_error (_("cannot open mailbox %s: %s"),
 	      path, mu_strerror (status));
@@ -448,8 +432,7 @@ notify_user (const char *user, const char *device, const char *path,
       return; /* FIXME: Notify the user, anyway */
     }
 
-  run_user_action (fp, cr, msg);
-  fclose (fp);
+  run_user_action (device, msg);
 }
 
 /* Search utmp for the local user */
@@ -473,7 +456,7 @@ find_user (const char *name, char *tty)
       if (uptr->ut_type != USER_PROCESS)
 	continue;
 #endif
-      if (!strncmp (uptr->ut_name, name, sizeof(uptr->ut_name)))
+      if (!strncmp (uptr->ut_user, name, sizeof(uptr->ut_user)))
 	{
 	  /* no particular tty was requested */
 	  strncpy (ftty + sizeof(PATH_DEV),
@@ -563,15 +546,13 @@ int
 main (int argc, char **argv)
 {
   int c;
-  int ind;
-
+  char **save_argv;
+  
   /* Native Language Support */
   MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
   comsat_init ();
   mu_acl_cfg_init ();
-  mu_m_server_cfg_init ();
   mu_m_server_create (&server, program_version);
   mu_m_server_set_type (server, MU_IP_UDP);
   mu_m_server_set_conn (server, comsat_connection);
@@ -580,60 +561,60 @@ main (int argc, char **argv)
   mu_m_server_set_max_children (server, 20);
   /* FIXME mu_m_server_set_pidfile (); */
   mu_m_server_set_default_port (server, 512);
-  /* FIXME: timeout is not needed. How to disable it? */
+  mu_m_server_cfg_init (server, NULL);
   
-  if (mu_app_init (&argp, comsat_argp_capa, comsat_cfg_param, argc, argv, 0,
-		   &ind, server))
-    exit (1);
+  /* FIXME: timeout is not needed. How to disable it? */
+  mu_log_syslog = 1;
+
+  save_argv = argv;
+  
+  mu_cli (argc, argv, &cli, capa, server, &argc, &argv);
 
   if (test_mode)
     {
+      struct passwd *pw;
       char *user;
       
-      argc -= ind;
-      argv += ind;
-  
+      mu_stdstream_strerr_setup (MU_STRERR_STDERR);
+      biffrc_errors = BIFFRC_ERRORS_TO_ERR;
       if (argc < 2 || argc > 2)
 	{
 	  mu_error (_("mailbox URL and message QID are required in test mode"));
 	  exit (EXIT_FAILURE);
 	}
 
-      user = getenv ("LOGNAME");
-      if (!user)
+      pw = getpwuid (getuid ());
+      if (!pw)
 	{
-	  user = getenv ("USER");
-	  if (!user)
-	    {
-	      struct passwd *pw = getpwuid (getuid ());
-	      if (!pw)
-		{
-		  mu_error (_("cannot determine user name"));
-		  exit (EXIT_FAILURE);
-		}
-	      user = pw->pw_name;
-	    }
+	  mu_error (_("cannot determine user name"));
+	  exit (EXIT_FAILURE);
 	}
-		  
+      user = pw->pw_name;
+
+      if (biffrc[0] == '.' && (biffrc[1] == '/' ||
+			       (biffrc[1] == '.' && biffrc[2] == '/')))
+	{
+	  char *cwd = mu_getcwd ();
+	  biffrc = mu_make_file_name (cwd, biffrc);
+	  if (!biffrc)
+	    {
+	      mu_error ("%s", mu_strerror (ENOMEM));
+	      exit (1);
+	    }
+	  mu_normalize_path (biffrc);
+	  free (cwd);
+	}
+      
       notify_user (user, "/dev/tty", argv[0], argv[1]);
       exit (0);
     }
   
-  /* Set up error messaging  */
-  openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
-
-  {
-    mu_debug_t debug;
-
-    mu_diag_get_debug (&debug);
-    mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
-
-    mu_debug_default_printer = mu_debug_syslog_printer;
-  }
+  mu_stdstream_strerr_setup (mu_log_syslog ?
+			     MU_STRERR_SYSLOG : MU_STRERR_STDERR);
 
   if (mu_m_server_mode (server) == MODE_DAEMON)
     {
-      if (argv[0][0] != '/')
+      if (save_argv[0][0] != '/')
 	mu_diag_output (MU_DIAG_NOTICE,
 			_("program name is not absolute; reloading will not "
 			  "be possible"));
@@ -654,7 +635,7 @@ main (int argc, char **argv)
       if (reload)
 	{
 	  mu_diag_output (MU_DIAG_NOTICE, _("restarting"));
-	  execvp (argv[0], argv);
+	  execvp (save_argv[0], save_argv);
 	}
     }
   else
@@ -665,4 +646,3 @@ main (int argc, char **argv)
   
   return c != 0;
 }
-
