@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2002, 2005, 2007, 2010-2012, 2014-2016 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -13,9 +13,8 @@
    GNU Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
-   License along with GNU Mailutils; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   License along with GNU Mailutils.  If not, see
+   <http://www.gnu.org/licenses/>. */; 
 
 /*  This is an example on how to write extension tests for GNU sieve.
     It provides test "numaddr".
@@ -79,48 +78,28 @@ _count_items (void *item, void *data)
 
 /* Handler for the numaddr test */
 static int
-numaddr_test (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+numaddr_test (mu_sieve_machine_t mach)
 {
-  mu_sieve_value_t *h, *v;
+  mu_sieve_value_t *h;
   struct val_ctr vc;
   int rc;
   
-  if (mu_sieve_get_debug_level (mach) & MU_SIEVE_DEBUG_TRACE)
-    {
-      mu_sieve_locus_t locus;
-      mu_sieve_get_locus (mach, &locus);
-      mu_sieve_debug (mach, "%s:%lu: NUMADDR\n",
-		   locus.source_file,
-		   (unsigned long) locus.source_line);
-    }
-
   /* Retrieve required arguments: */
   /* First argument: list of header names */
-  h = mu_sieve_value_get (args, 0);
-  if (!h)
-    {
-      mu_sieve_error (mach, "numaddr: can't get argument 1");
-      mu_sieve_abort (mach);
-    }
+  h = mu_sieve_get_arg_untyped (mach, 0);
   /* Second argument: Limit on the number of addresses */
-  v = mu_sieve_value_get (args, 1);
-  if (!v)
-    {
-      mu_sieve_error (mach, "numaddr: can't get argument 2");
-      mu_sieve_abort (mach);
-    }
+  mu_sieve_get_arg (mach, 1, SVT_NUMBER, &vc.limit);
 
   /* Fill in the val_ctr structure */
   mu_message_get_header (mu_sieve_get_message (mach), &vc.hdr);
   vc.count = 0;
-  vc.limit = v->v.number;
 
   /* Count the addresses */
-  rc = mu_sieve_vlist_do (h, _count_items, &vc);
+  rc = mu_sieve_vlist_do (mach, h, _count_items, &vc);
 
   /* Here rc >= 1 iff the counted number of addresses is greater or equal
      to vc.limit. If `:under' tag was given we reverse the return value */
-  if (mu_sieve_tag_lookup (tags, "under", NULL))
+  if (mu_sieve_get_tag (mach, "under", SVT_VOID, NULL))
     rc = !rc;
 
   return rc;
@@ -152,6 +131,7 @@ static mu_sieve_tag_group_t numaddr_tag_groups[] = {
 int
 SIEVE_EXPORT(numaddr,init) (mu_sieve_machine_t mach)
 {
-  return mu_sieve_register_test (mach, "numaddr", numaddr_test,
-                              numaddr_req_args, numaddr_tag_groups, 1);
+  mu_sieve_register_test (mach, "numaddr", numaddr_test,
+			  numaddr_req_args, numaddr_tag_groups, 1);
+  return 0;
 }
