@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007, 2008, 2009, 2010, 2011 Free Software
+   Copyright (C) 1999, 2001, 2007-2012, 2014-2016 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -29,11 +29,11 @@
                BAD - command unknown or arguments invalid
 */  
 int
-imap4d_delete (struct imap4d_command *command, imap4d_tokbuf_t tok)
+imap4d_delete (struct imap4d_session *session,
+               struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   int rc = RESP_OK;
   const char *msg = "Completed";
-  const char *delim = "/";
   char *name;
   mu_mailbox_t tmpbox;
   
@@ -49,7 +49,7 @@ imap4d_delete (struct imap4d_command *command, imap4d_tokbuf_t tok)
     return io_completion_response (command, RESP_NO, "Already exist");
 
  /* Allocates memory.  */
-  name = namespace_getfullpath (name, delim, NULL);
+  name = namespace_getfullpath (name, NULL);
   if (!name)
     return io_completion_response (command, RESP_NO, "Cannot remove");
 
