@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2012 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2012, 2014-2016 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +16,9 @@
    along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
-#ifdef WITH_GSASL
-# include <mailutils/gsasl.h>
-#endif
-#include "mailutils/libargp.h"
+#include <mailutils/gsasl.h>
+#include "mailutils/cli.h"
+#include "mailutils/kwd.h"
 #include "tcpwrap.h"
 
 mu_m_server_t server;
@@ -32,10 +32,12 @@ char *modify_homedir;           /* Expression to produce imap4d_homedir */
 int state = STATE_NONAUTH;      /* Current IMAP4 state */
 struct mu_auth_data *auth_data; 
 
+enum tls_mode tls_mode;
 int login_disabled;             /* Disable LOGIN command */
-int tls_required;               /* Require STARTTLS */
 int create_home_dir;            /* Create home directory if it does not
 				   exist */
+mu_list_t user_retain_groups;
+
 int home_dir_mode = S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
 
 int mailbox_mode[NS_MAX];
@@ -54,94 +56,79 @@ int ident_encrypt_only;
 int test_mode;
 
 const char *program_version = "imap4d (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU imap4d -- the IMAP4D daemon.");
 
-#define OPT_PREAUTH         259
-#define OPT_FOREGROUND      260
-#define OPT_TEST_MODE       261
+
+static void
+set_foreground (struct mu_parseopt *po, struct mu_option *opt,
+		char const *arg)
+{
+  mu_m_server_set_foreground (server, 1);
+}
 
-static struct argp_option options[] = {
-  { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), 0},
-  { "inetd",  'i', 0, 0, N_("run in inetd mode"), 0},
-  { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("runs in daemon mode with a maximum of NUMBER children"), 0 },
-  { "test", OPT_TEST_MODE, 0, 0,
-    N_("run in test mode"), 0 },
-  { "preauth", OPT_PREAUTH, NULL, 0,
-    N_("start in preauth mode") },
+static void
+set_inetd_mode (struct mu_parseopt *po, struct mu_option *opt,
+		char const *arg)
+{
+  mu_m_server_set_mode (server, MODE_INTERACTIVE);
+}
   
-  {NULL, 0, NULL, 0, NULL, 0}
-};
+static void
+set_daemon_mode (struct mu_parseopt *po, struct mu_option *opt,
+		 char const *arg)
+{
+  mu_m_server_set_mode (server, MODE_DAEMON);
+  if (arg)
+    {
+      size_t max_children;
+      char *errmsg;
+      int rc = mu_str_to_c (arg, mu_c_size, &max_children, &errmsg);
+      if (rc)
+	{
+	  mu_parseopt_error (po, _("%s: bad argument"), arg);
+	  exit (po->po_exit_error);
+	}
+      mu_m_server_set_max_children (server, max_children);
+    }
+}
+
+static void
+set_preauth (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
+{
+  preauth_mode = preauth_stdio;
+}
+
+static struct mu_option imap4d_options[] = {
+  { "foreground",  0, NULL, MU_OPTION_DEFAULT,
+    N_("remain in foreground"),
+    mu_c_bool, NULL, set_foreground },
+  { "inetd",  'i', NULL, MU_OPTION_DEFAULT,
+    N_("run in inetd mode"),
+    mu_c_bool, NULL, set_inetd_mode },
+  { "daemon", 'd', N_("NUMBER"), MU_OPTION_ARG_OPTIONAL,
+    N_("runs in daemon mode with a maximum of NUMBER children"),
+    mu_c_string, NULL, set_daemon_mode },
+
+  { "test", 0, NULL, MU_OPTION_DEFAULT,
+    N_("run in test mode"),
+    mu_c_bool, &test_mode },
+
+  { "preauth", 0, NULL, MU_OPTION_DEFAULT,
+    N_("start in preauth mode"),
+    mu_c_string, NULL, set_preauth },
+
+  MU_OPTION_END
+}, *options[] = { imap4d_options, NULL };
 
 
-static error_t imap4d_parse_opt (int key, char *arg,
-				 struct argp_state *state);
-
-static struct argp argp = {
-  options,
-  imap4d_parse_opt,
-  NULL,
-  doc,
-  NULL,
-  NULL, NULL
-};
-
-static const char *imap4d_capa[] = {
+static char *capa[] = {
   "auth",
-  "common",
   "debug",
   "mailbox",
   "locking",
   "logging",
   NULL
 };
-
-static int imap4d_mainloop (int, int, int);
-
-static error_t
-imap4d_parse_opt (int key, char *arg, struct argp_state *state)
-{
-  static mu_list_t lst;
-
-  switch (key)
-    {
-    case 'd':
-      mu_argp_node_list_new (lst, "mode", "daemon");
-      if (arg)
-	mu_argp_node_list_new (lst, "max-children", arg);
-      break;
-
-    case 'i':
-      mu_argp_node_list_new (lst, "mode", "inetd");
-      break;
-
-    case OPT_FOREGROUND:
-      mu_argp_node_list_new (lst, "foreground", "yes");
-      break;
-      
-    case OPT_PREAUTH:
-      preauth_mode = preauth_stdio;
-      break;
-
-    case OPT_TEST_MODE:
-      mu_argp_node_list_new (lst, "mode", "inetd");
-      test_mode = 1;
-      break;
-      
-    case ARGP_KEY_INIT:
-      mu_argp_node_list_init (&lst);
-      break;
-      
-    case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (lst, NULL, NULL);
-      break;
-      
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
-}
-
+
 static int
 cb_mode (void *data, mu_config_value_t *val)
 {
@@ -208,7 +195,7 @@ cb_preauth (void *data, mu_config_value_t *val)
     return parse_preauth_scheme (val->v.string, NULL);
   else if (val->v.string[0] == '/')
     {
-      preauth_program = xstrdup (val->v.string);
+      preauth_program = mu_strdup (val->v.string);
       preauth_mode = preauth_prog;
     }
   else
@@ -249,75 +236,297 @@ cb_mailbox_mode (void *data, mu_config_value_t *val)
     mu_error (_("invalid mode string near %s"), p);
   return 0;
 }
+
+static int
+cb2_group (const char *gname, void *data)
+{
+  mu_list_t list = data;
+  struct group *group;
+  
+  group = getgrnam (gname);
+  if (!group)
+    mu_error (_("unknown group: %s"), gname);
+  else
+    mu_list_append (list, (void*) (intptr_t) group->gr_gid);
+  return 0;
+}
+  
+static int
+cb_group (void *data, mu_config_value_t *arg)
+{
+  mu_list_t *plist = data;
 
+  if (!*plist)
+    mu_list_create (plist);
+  return mu_cfg_string_value_cb (arg, cb2_group, *plist);
+}
+
+
 struct imap4d_srv_config
 {
   struct mu_srv_config m_cfg;
-  int tls;
+  enum tls_mode tls_mode;
 };
 
+#ifdef WITH_TLS
+static int
+cb_tls (void *data, mu_config_value_t *val)
+{
+  int *res = data;
+  static struct mu_kwd tls_kwd[] = {
+    { "no", tls_no },
+    { "false", tls_no },
+    { "off", tls_no },
+    { "0", tls_no },
+    { "ondemand", tls_ondemand },
+    { "stls", tls_ondemand },
+    { "required", tls_required },
+    { "connection", tls_connection },
+    /* For compatibility with prior versions: */
+    { "yes", tls_connection }, 
+    { "true", tls_connection },
+    { "on", tls_connection },
+    { "1", tls_connection },
+    { NULL }
+  };
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+
+  if (mu_kwd_xlat_name (tls_kwd, val->v.string, res))
+    mu_error (_("not a valid tls keyword: %s"), val->v.string);
+  return 0;
+}
+#endif
+
+#ifdef WITH_TLS
+static int
+cb_tls_required (void *data, mu_config_value_t *val)
+{
+  int bv;
+
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  if (mu_str_to_c (val->v.string, mu_c_bool, &bv, NULL))
+    mu_error (_("Not a boolean value"));
+  else if (bv)
+    {
+      tls_mode = tls_required;
+      mu_diag_output (MU_DIAG_WARNING,
+		      "the \"tls-required\" statement is deprecated, "
+		      "use \"tls required\" instead");
+    }
+  else
+    mu_diag_output (MU_DIAG_WARNING,
+		    "the \"tls-required\" statement is deprecated, "
+		    "use \"tls\" instead");
+    
+  return 0;
+}
+#endif
+
+static mu_list_t auth_deny_user_list, auth_allow_user_list;
+static mu_list_t auth_deny_group_list, auth_allow_group_list;
+
+static int
+check_user_groups (void *item, void *data)
+{
+  char *gname = item;
+  struct group *gp;
+  char **p;
+  
+  gp = getgrnam (gname);
+  if (!gp)
+    return 0;
+
+  if (gp->gr_gid == auth_data->gid)
+    return MU_ERR_EXISTS;
+
+  for (p = gp->gr_mem; *p; p++)
+    if (strcmp (*p, auth_data->name) == 0)
+      return MU_ERR_EXISTS;
+
+  return 0;
+}
+
+static int
+imap_check_group_list (mu_list_t l)
+{
+  int rc = mu_list_foreach (l, check_user_groups, NULL);
+  if (rc == MU_ERR_EXISTS) 
+    return 0;
+  else if (rc == 0)
+    return MU_ERR_NOENT;
+  return rc;
+}
+
 static struct mu_cfg_param imap4d_srv_param[] = {
-  { "tls", mu_cfg_bool, NULL, mu_offsetof (struct imap4d_srv_config, tls),
-    NULL,
-    N_("Use TLS encryption for this server")
-  },
+#ifdef WITH_TLS
+  { "tls", mu_cfg_callback,
+    NULL, mu_offsetof (struct imap4d_srv_config, tls_mode), cb_tls,
+    N_("Kind of TLS encryption to use for this server"),
+    /* TRANSLATORS: translate only arg:, the rest are keywords */
+    N_("arg: false|true|ondemand|stls|requred|connection") },
+#endif
   { NULL }
 };
 
 static struct mu_cfg_param imap4d_cfg_param[] = {
-  { "homedir", mu_cfg_string, &modify_homedir, 0, NULL,
+  { "allow-users", MU_CFG_LIST_OF(mu_c_string), &auth_allow_user_list,
+    0, NULL, 
+    N_("Allow access to users from this list.") },
+  { "deny-users", MU_CFG_LIST_OF(mu_c_string), &auth_deny_user_list,
+    0, NULL, 
+    N_("Deny access to users from this list.") },
+  { "allow-groups", MU_CFG_LIST_OF(mu_c_string), &auth_allow_group_list,
+    0, NULL, 
+    N_("Allow access if the user group is in this list.") },
+  { "deny-groups", MU_CFG_LIST_OF(mu_c_string), &auth_deny_group_list,
+    0, NULL, 
+    N_("Deny access if the user group is in this list.") },
+  
+  { "homedir", mu_c_string, &modify_homedir, 0, NULL,
     N_("Modify home directory.") },
-  { "personal-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_PRIVATE],
+  { "personal-namespace", MU_CFG_LIST_OF(mu_c_string), &namespace[NS_PRIVATE],
     0, NULL, 
     N_("Set personal namespace.") },
-  { "other-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_OTHER],
+  { "other-namespace", MU_CFG_LIST_OF(mu_c_string), &namespace[NS_OTHER],
     0, NULL, 
     N_("Set other users' namespace.") },
-  { "shared-namespace", MU_CFG_LIST_OF(mu_cfg_string), &namespace[NS_SHARED],
+  { "shared-namespace", MU_CFG_LIST_OF(mu_c_string), &namespace[NS_SHARED],
     0, NULL,
     N_("Set shared namespace.") },
   { "other-mailbox-mode", mu_cfg_callback, &mailbox_mode[NS_OTHER], 0,
     cb_mailbox_mode,
-    N_("File mode for mailboxes in other namespace.") },
+    N_("File mode for mailboxes in other namespace."),
+    N_("mode: g(+|=)[wr]+,o(+|=)[wr]+") },
   { "shared-mailbox-mode", mu_cfg_callback, &mailbox_mode[NS_SHARED], 0,
     cb_mailbox_mode,
-    N_("File mode for mailboxes in shared namespace.") },
-  { "login-disabled", mu_cfg_bool, &login_disabled, 0, NULL,
+    N_("File mode for mailboxes in shared namespace."),
+    N_("mode: g(+|=)[wr]+,o(+|=)[wr]+") },
+  { "login-disabled", mu_c_bool, &login_disabled, 0, NULL,
     N_("Disable LOGIN command.") },
-  { "create-home-dir", mu_cfg_bool, &create_home_dir, 0, NULL,
+  { "create-home-dir", mu_c_bool, &create_home_dir, 0, NULL,
     N_("If true, create non-existing user home directories.") },
   { "home-dir-mode", mu_cfg_callback, NULL, 0, cb_mode,
-    N_("File mode for creating user home directories (octal)."),
-    N_("mode") },
-  { "tls-required", mu_cfg_bool, &tls_required, 0, NULL,
-    N_("Always require STARTTLS before entering authentication phase.") },
+    N_("File mode for creating user home directories."),
+    N_("mode: octal") },
+  { "retain-groups", mu_cfg_callback, &user_retain_groups, 0, cb_group,
+    N_("Retain these supplementary groups when switching to user privileges"),
+    N_("groups: list of string") },
+#ifdef WITH_TLS
+  { "tls", mu_cfg_callback, &tls_mode, 0, cb_tls,
+    N_("Kind of TLS encryption to use"),
+    /* TRANSLATORS: translate only arg:, the rest are keywords */
+    N_("arg: false|true|ondemand|stls|requred|connection") },
+  { "tls-required", mu_cfg_callback, &tls_mode, 0, cb_tls_required,
+    N_("Always require STLS before entering authentication phase.\n"
+       "Deprecated, use \"tls required\" instead."),
+    N_("arg: bool") },
+#endif
   { "preauth", mu_cfg_callback, NULL, 0, cb_preauth,
-    N_("Configure PREAUTH mode.  MODE is one of:\n"
+    N_("Configure PREAUTH mode.  <value> is one of:\n"
        "  prog:///<full-program-name: string>\n"
        "  ident[://:<port: string-or-number>]\n"
        "  stdio"),
-    N_("mode") },
-  { "preauth-only", mu_cfg_bool, &preauth_only, 0, NULL,
+    N_("mode: value") },
+  { "preauth-only", mu_c_bool, &preauth_only, 0, NULL,
     N_("Use only preauth mode.  If unable to setup it, disconnect "
        "immediately.") },
-  { "ident-keyfile", mu_cfg_string, &ident_keyfile, 0, NULL,
-    N_("Name of DES keyfile for decoding ecrypted ident responses.") },
-  { "ident-encrypt-only", mu_cfg_bool, &ident_encrypt_only, 0, NULL,
+  { "ident-keyfile", mu_c_string, &ident_keyfile, 0, NULL,
+    N_("Name of DES keyfile for decoding encrypted ident responses.") },
+  { "ident-encrypt-only", mu_c_bool, &ident_encrypt_only, 0, NULL,
     N_("Use only encrypted ident responses.") },
-  { "id-fields", MU_CFG_LIST_OF(mu_cfg_string), &imap4d_id_list, 0, NULL,
+  { "id-fields", MU_CFG_LIST_OF(mu_c_string), &imap4d_id_list, 0, NULL,
     N_("List of fields to return in response to ID command.") },
   { "mandatory-locking", mu_cfg_section },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("Server configuration.") },
-  { "transcript", mu_cfg_bool, &imap4d_transcript, 0, NULL,
+  { "transcript", mu_c_bool, &imap4d_transcript, 0, NULL,
     N_("Set global transcript mode.") },
   TCP_WRAPPERS_CONFIG
   { NULL }
 };
 
+struct mu_cli_setup cli = {
+  .optv = options,
+  .cfg = imap4d_cfg_param,
+  .prog_doc = N_("GNU imap4d -- the IMAP4D daemon."),
+  .server = 1
+};
+
+int
+mu_get_user_groups (const char *user, mu_list_t retain, mu_list_t *pgrouplist)
+{
+  int rc;
+  struct group *gr;
+  mu_list_t list;
+	
+  if (!*pgrouplist)
+    {
+      rc = mu_list_create (pgrouplist);
+      if (rc)
+	{
+	  mu_error(_("%s: cannot create list: %s"),
+		   "mu_get_user_groups", mu_strerror (rc));
+	  return rc;
+	}
+    }
+
+  list = *pgrouplist;
+  setgrent ();
+  for (rc = 0; rc == 0 && (gr = getgrent ()); )
+    {
+      char **p;
+      for (p = gr->gr_mem; *p; p++)
+	if (strcmp (*p, user) == 0)
+	  {
+	    if (retain
+		&& mu_list_locate (retain, (void*) (intptr_t) gr->gr_gid,
+				   NULL))
+	      continue;
+	      
+	    /* FIXME: Avoid duplicating gids */
+	    rc = mu_list_append (list, (void*) (intptr_t) gr->gr_gid);
+	    if (rc) 
+	      mu_error(_("%s: cannot append to list: %s"),
+		       "mu_get_user_groups",
+		       mu_strerror (rc));
+	    break;
+	  }
+    }
+  endgrent ();
+  return rc;
+}
+
 int
 imap4d_session_setup0 ()
 {
+  if (auth_deny_user_list &&
+      mu_list_locate (auth_deny_user_list, auth_data->name, NULL) == 0)
+    {
+      mu_error (_("%s is in deny-users, rejecting"), auth_data->name);
+      return 1;
+    }
+  if (auth_allow_user_list &&
+      mu_list_locate (auth_allow_user_list, auth_data->name, NULL))
+    {
+      mu_error (_("%s is not in allow-users, rejecting"), auth_data->name);
+      return 1;
+    }
+  if (auth_deny_group_list &&
+      imap_check_group_list (auth_deny_group_list) == 0)
+    {
+      mu_error (_("%s is in deny-groups, rejecting"), auth_data->name);
+      return 1;
+    }
+  if (auth_allow_group_list &&
+      imap_check_group_list (auth_allow_group_list))
+    {
+      mu_error (_("%s is not in allow-groups, rejecting"), auth_data->name);
+      return 1;
+    }
+
   real_homedir = mu_normalize_path (mu_strdup (auth_data->dir));
   if (imap4d_check_home_dir (real_homedir, auth_data->uid, auth_data->gid))
     return 1;
@@ -349,7 +558,7 @@ imap4d_session_setup0 ()
 	  mu_error (_("expanding %s yields empty string"), expr);
 	  return 1;
 	}
-      imap4d_homedir = strdup (ws.ws_wordv[0]);
+      imap4d_homedir = mu_strdup (ws.ws_wordv[0]);
       if (!imap4d_homedir)
 	{
 	  mu_error ("%s", mu_strerror (errno));
@@ -357,7 +566,7 @@ imap4d_session_setup0 ()
 	}
     }
   else
-    imap4d_homedir = strdup (real_homedir);
+    imap4d_homedir = mu_strdup (real_homedir);
 
   if (strcmp (imap4d_homedir, real_homedir)
       && imap4d_check_home_dir (imap4d_homedir,
@@ -369,7 +578,48 @@ imap4d_session_setup0 ()
     }
   
   if (auth_data->change_uid)
-    setuid (auth_data->uid);
+    {
+      struct group *gr;
+      mu_list_t groups = NULL;
+      int rc;
+      uid_t uid;
+
+      uid = getuid ();
+      if (uid != auth_data->uid)
+	{
+	  rc = mu_list_create (&groups);
+	  if (rc)
+	    {
+	      mu_error(_("cannot create list: %s"), mu_strerror (rc));
+	      free (imap4d_homedir);
+	      free (real_homedir);
+	      return 1;
+	    }
+	  mu_list_append (groups, (void*) (intptr_t) auth_data->gid);
+
+	  rc = mu_get_user_groups (auth_data->name, user_retain_groups,
+				   &groups);
+	  if (rc)
+	    {
+	      /* FIXME: When mu_get_user_groups goes to the library, add a
+		 diag message here */
+	      free (imap4d_homedir);
+	      free (real_homedir);
+	      return 1;
+	    }
+	  gr = getgrnam ("mail");
+	  rc = mu_switch_to_privs (auth_data->uid, gr->gr_gid, groups);
+	  mu_list_destroy (&groups);
+	  if (rc)
+	    {
+	      mu_error (_("can't switch to user %s privileges: %s"),
+			auth_data->name, mu_strerror (rc));
+	      free (imap4d_homedir);
+	      free (real_homedir);
+	      return 1;
+	    }
+	}
+    }
 
   util_chdir (imap4d_homedir);
   namespace_init_session (imap4d_homedir);
@@ -410,12 +660,12 @@ get_client_address (int fd, struct sockaddr_in *pcs)
 }
 
 static int
-set_strerr_flt ()
+set_strerr_flt (void)
 {
   mu_stream_t flt, trans[2];
   int rc;
   
-  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
   if (rc == 0)
     {
       char sessidstr[10];
@@ -432,7 +682,7 @@ set_strerr_flt ()
 	  mu_stream_set_buffer (flt, mu_buffer_line, 0);
 	  trans[0] = flt;
 	  trans[1] = NULL;
-	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM,
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM,
 				MU_IOCTL_OP_SET, trans);
 	  mu_stream_unref (trans[0]);
 	  if (rc)
@@ -451,21 +701,21 @@ set_strerr_flt ()
 }
 
 static void
-clr_strerr_flt ()
+clr_strerr_flt (void)
 {
   mu_stream_t flt, trans[2];
   int rc;
 
-  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
   if (rc == 0)
     {
       flt = trans[0];
 
-      rc = mu_stream_ioctl (flt, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+      rc = mu_stream_ioctl (flt, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
       if (rc == 0)
 	{
 	  mu_stream_unref (trans[0]);
-	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM,
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM,
 				MU_IOCTL_OP_SET, trans);
 	  if (rc == 0)
 	    mu_stream_unref (flt);
@@ -482,11 +732,12 @@ imap4d_child_signal_setup (RETSIGTYPE (*handler) (int signo))
 }
 
 static int
-imap4d_mainloop (int ifd, int ofd, int tls)
+imap4d_mainloop (int ifd, int ofd, enum tls_mode tls)
 {
   imap4d_tokbuf_t tokp;
   char *text;
   int signo;
+  struct imap4d_session session;
 
   if (!test_mode)
     test_mode = isatty (ifd);
@@ -523,7 +774,33 @@ imap4d_mainloop (int ifd, int ofd, int tls)
       imap4d_child_signal_setup (imap4d_child_signal);
     }
   
-  io_setio (ifd, ofd, tls);
+  if (tls == tls_unspecified)
+    tls = tls_available ? tls_ondemand : tls_no;
+  else if (tls != tls_no && !tls_available)
+    {
+      mu_error (_("TLS is not configured, but requested in the "
+		  "configuration"));
+      tls = tls_no;
+    }
+
+  switch (tls)
+    {
+    case tls_required:
+      imap4d_capability_add (IMAP_CAPA_XTLSREQUIRED);
+      break;
+    case tls_no:
+      imap4d_capability_remove (IMAP_CAPA_STARTTLS);
+      tls_available = 0;
+      break;
+    default:
+      break;
+    }
+  
+  session.tls_mode = tls;
+  
+  io_setio (ifd, ofd, tls == tls_connection);
+  if (tls == tls_connection)
+    tls_encryption_on (&session);
 
   if (imap4d_preauth_setup (ifd) == 0)
     {
@@ -557,7 +834,7 @@ imap4d_mainloop (int ifd, int ofd, int tls)
       imap4d_readline (tokp);
       /* check for updates */
       imap4d_sync ();
-      util_do_command (tokp);
+      util_do_command (&session, tokp);
       imap4d_sync ();
       io_flush ();
     }
@@ -580,7 +857,8 @@ imap4d_connection (int fd, struct sockaddr *sa, int salen,
   else
     rc = 1;
   
-  imap4d_mainloop (fd, fd, cfg->tls);
+  imap4d_mainloop (fd, fd, 
+                   cfg->tls_mode == tls_unspecified ? tls_mode : cfg->tls_mode);
 
   if (rc == 0)
     clr_strerr_flt ();
@@ -627,6 +905,12 @@ imap4d_master_signal (int signo)
   longjmp (master_jmp, signo);
 }
 
+static void
+imap4d_alloc_die ()
+{
+  imap4d_bye (ERR_NO_MEM);
+}
+
 
 int
 main (int argc, char **argv)
@@ -649,19 +933,10 @@ main (int argc, char **argv)
   mu_register_local_mbox_formats ();
   
   imap4d_capability_init ();
-#ifdef WITH_TLS
-  mu_gocs_register ("tls", mu_tls_module_init);
-#endif /* WITH_TLS */
-#ifdef WITH_GSASL
-  mu_gocs_register ("gsasl", mu_gsasl_module_init);
-#endif
   mu_tcpwrapper_cfg_init ();
   manlock_cfg_init ();
   mu_acl_cfg_init ();
-  mu_m_server_cfg_init (imap4d_srv_param);
   
-  mu_argp_init (NULL, NULL);
-
   mu_m_server_create (&server, program_version);
   mu_m_server_set_config_size (server, sizeof (struct imap4d_srv_config));
   mu_m_server_set_conn (server, imap4d_connection);
@@ -672,19 +947,24 @@ main (int argc, char **argv)
   mu_m_server_set_default_port (server, 143);
   mu_m_server_set_timeout (server, 1800);  /* RFC2060: 30 minutes. */
   mu_m_server_set_strexit (server, mu_strexit);
+  mu_m_server_cfg_init (server, imap4d_srv_param);
   
+  mu_alloc_die_hook = imap4d_alloc_die;
+
   mu_log_syslog = 1;
 
-  if (mu_app_init (&argp, imap4d_capa, imap4d_cfg_param, 
-		   argc, argv, 0, NULL, server))
-    exit (EX_CONFIG); /* FIXME: No way to discern from EX_USAGE? */
-
+  mu_cli (argc, argv, &cli, capa, server, &argc, &argv);
+  if (argc)
+    {
+      mu_error (_("too many arguments"));
+      exit (EX_USAGE);
+    }
+  
+  if (test_mode)
+    mu_m_server_set_mode (server, MODE_INTERACTIVE);
+  
   if (login_disabled)
     imap4d_capability_add (IMAP_CAPA_LOGINDISABLED);
-#ifdef WITH_TLS
-  if (tls_required)
-    imap4d_capability_add (IMAP_CAPA_XTLSREQUIRED);
-#endif
 
   namespace_init ();
 
@@ -772,7 +1052,7 @@ main (int argc, char **argv)
     {
       /* Make sure we are in the root directory.  */
       chdir ("/");
-      status = imap4d_mainloop (MU_STDIN_FD, MU_STDOUT_FD, 0);
+      status = imap4d_mainloop (MU_STDIN_FD, MU_STDOUT_FD, tls_mode);
     }
 
   if (status)
