@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2009, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2016 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 /**
 * Parses syntatic elements defined in RFC 822.
@@ -25,7 +24,7 @@
 #define _MAILUTILS_PARSE822_H
 
 #include <mailutils/types.h>
-#include <mailutils/mutil.h>
+#include <mailutils/datetime.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -132,7 +131,7 @@ extern int mu_parse822_date      (const char **p, const char *e, int *day,
 extern int mu_parse822_time      (const char **p, const char *e, int *h,
 				  int *m, int *s, int *tz, const char **tz_name);
 extern int mu_parse822_date_time (const char **p, const char *e,
-				  struct tm *tm, mu_timezone *tz);
+				  struct tm *tm, struct mu_timezone *tz);
 
 
 #ifdef __cplusplus
