@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2007-2008, 2010-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003, 2005, 2007-2008, 2010-2012, 2014-2016 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 #define _MAILUTILS_TLS_H
 
 #include <mailutils/types.h>
+#include <mailutils/cli.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -37,9 +38,9 @@ struct mu_tls_module_config
   
   char *ssl_cafile;
   int ssl_cafile_safety_checks;
-};
 
-extern int mu_tls_module_init (enum mu_gocs_op, void *);
+  char *priorities;
+};
 
 extern int mu_tls_server_stream_create (mu_stream_t *stream, 
 			 	        mu_stream_t strin, mu_stream_t strout,
@@ -53,6 +54,8 @@ extern int mu_init_tls_libs (int x509);
 extern void mu_deinit_tls_libs (void);
 
 extern int mu_tls_enable;
+
+extern struct mu_cli_capa mu_cli_capa_tls;
   
 #ifdef __cplusplus
 }
