@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-   2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2012, 2014-2016 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -63,6 +62,7 @@
 #include <mailutils/locker.h>
 #include <mailutils/message.h>
 #include <mailutils/util.h>
+#include <mailutils/datetime.h>
 #include <mailutils/property.h>
 #include <mailutils/stream.h>
 #include <mailutils/url.h>
@@ -216,34 +216,6 @@ _amd_prop_store_off (struct _amd_data *amd, const char *name, mu_off_t val)
   return mu_property_set_value (amd->prop, name, p, 1);
 }
 
-/* Backward-compatible size file support */
-static int
-read_size_file (struct _amd_data *amd)
-{
-  FILE *fp;
-  int rc;
-  char *name = mu_make_file_name (amd->name, _MU_AMD_SIZE_FILE_NAME);
-  if (!name)
-    return 1;
-  fp = fopen (name, "r");
-  if (fp)
-    {
-      unsigned long size;
-      if (fscanf (fp, "%lu", &size) == 1)
-	{
-	  rc = _amd_prop_store_off (amd, _MU_AMD_PROP_SIZE, size);
-	}
-      else
-	rc = 1;
-      fclose (fp);
-      unlink (name);
-    }
-  else
-    rc = 1;
-  free (name);
-  return rc;
-}
-
 static int
 _amd_prop_create (struct _amd_data *amd)
 {
@@ -267,8 +239,7 @@ _amd_prop_create (struct _amd_data *amd)
       free (mhprop->filename);
       free (mhprop);
     }
-  else
-    read_size_file (amd);
+  
   return rc;
 }
 
@@ -548,6 +519,8 @@ amd_close (mu_mailbox_t mailbox)
   free (amd->msg_array);
   amd->msg_array = NULL;
 
+  mu_property_save (amd->prop);
+    
   amd->msg_count = 0; /* number of messages in the list */
   amd->msg_max = 0;   /* maximum message buffer capacity */
 
@@ -695,19 +668,17 @@ static int
 _amd_scan0 (struct _amd_data *amd, size_t msgno, size_t *pcount,
 	    int do_notify)
 {
+  unsigned long uidval;
   int status = amd->scan0 (amd->mailbox, msgno, pcount, do_notify);
   if (status != 0)
     return status;
   /* Reset the uidvalidity.  */
-  if (amd->msg_count > 0)
+  if (amd->msg_count == 0 ||
+      _amd_prop_fetch_ulong (amd, _MU_AMD_PROP_UIDVALIDITY, &uidval) ||
+      !uidval)
     {
-      unsigned long uidval;
-      if (_amd_prop_fetch_ulong (amd, _MU_AMD_PROP_UIDVALIDITY, &uidval) ||
-	  !uidval)
-	{
-	  uidval = (unsigned long)time (NULL);
-	  _amd_prop_store_off (amd, _MU_AMD_PROP_UIDVALIDITY, uidval);
-	}
+      uidval = (unsigned long) amd->mtime;
+      _amd_prop_store_off (amd, _MU_AMD_PROP_UIDVALIDITY, uidval);
     }
   return 0;
 }
@@ -722,7 +693,7 @@ amd_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
   /* Sanity checks.  */
   if (pmsg == NULL)
     return MU_ERR_OUT_PTR_NULL;
-  if (amd == NULL)
+  if (amd == NULL || msgno < 1)
     return EINVAL;
 
   /* If we did not start a scanning yet do it now.  */
@@ -734,7 +705,7 @@ amd_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
     }
 
   if ((mhm = _amd_get_message (amd, msgno)) == NULL)
-    return EINVAL;
+    return MU_ERR_NOENT;
   return _amd_attach_message (mailbox, mhm, pmsg);
 }
 
@@ -1218,7 +1189,7 @@ _compute_mailbox_size_recursive (struct _amd_data *amd, const char *name,
   free (buf);
   
   closedir (dir);
-  return 0;
+  return status;
 }
 
 static int
@@ -1248,20 +1219,11 @@ amd_remove_mbox (mu_mailbox_t mailbox)
     {
       char *name;
 
-      name = mu_make_file_name (amd->name, _MU_AMD_SIZE_FILE_NAME);
+      name = mu_make_file_name (amd->name, _MU_AMD_PROP_FILE_NAME);
       if (!name)
 	return ENOMEM;
       if (unlink (name) && errno != ENOENT)
 	rc = errno;
-      else
-	{
-	  free (name);
-	  name = mu_make_file_name (amd->name, _MU_AMD_PROP_FILE_NAME);
-	  if (!name)
-	    return ENOMEM;
-	  if (unlink (name) && errno != ENOENT)
-	    rc = errno;
-	}
       free (name);
     }
 
@@ -1290,7 +1252,9 @@ _amd_update_message (struct _amd_data *amd, struct _amd_message *mhm,
     flg = mu_message_is_modified (mhm->message);
   else if (mhm->attr_flags & MU_ATTRIBUTE_MODIFIED)
     flg = MU_MSG_ATTRIBUTE_MODIFIED;
-
+  else
+    return 0;
+  
   if (!flg)
     return 0;
 
@@ -1502,7 +1466,7 @@ amd_uidvalidity (mu_mailbox_t mailbox, unsigned long *puidvalidity)
   int status = amd_messages_count (mailbox, NULL);
   if (status != 0)
     return status;
-  /* If we did not start a scanning yet do it now.  */
+  /* If we did not start scanning yet do it now.  */
   if (amd->msg_count == 0)
     {
       status = _amd_scan0 (amd, 1, NULL, 0);
