@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007, 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2009-2012, 2014-2016 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -452,13 +453,13 @@ destroy (mu_iterator_t iterator, void *data)
 }
 
 static int
-curitem_p (void *owner, void *item)
+delitem (void *owner, void *item)
 {
   struct assoc_iterator *itr = owner;
   mu_assoc_t assoc = itr->assoc;
   struct _mu_assoc_elem *elem = ASSOC_ELEM (assoc, itr->index);
   
-  return elem == item;
+  return elem == item ? MU_ITR_DELITEM_NEXT : MU_ITR_DELITEM_NOTHING;
 }
 
 static int
@@ -498,7 +499,7 @@ mu_assoc_get_iterator (mu_assoc_t assoc, mu_iterator_t *piterator)
   mu_iterator_set_next (iterator, next);
   mu_iterator_set_getitem (iterator, getitem);
   mu_iterator_set_finished_p (iterator, finished_p);
-  mu_iterator_set_curitem_p (iterator, curitem_p);
+  mu_iterator_set_delitem (iterator, delitem);
   mu_iterator_set_destroy (iterator, destroy);
   mu_iterator_set_dup (iterator, assoc_data_dup);
   
