@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007-2012, 2014-2016 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,22 +13,24 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
+#include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <ctype.h>
 
 #include <mailutils/mu_auth.h>
 #include <mailutils/cstr.h>
+#include <mailutils/io.h>
 
 #ifdef WITH_LDAP
 #include "mailutils/argcv.h"
+#include "mailutils/wordsplit.h"
 #include "mailutils/assoc.h"
 #include "mailutils/list.h"
 #include "mailutils/iterator.h"
@@ -37,13 +40,12 @@
 #include "mailutils/error.h"
 #include "mailutils/errno.h"
 #include "mailutils/nls.h"
-#include "mailutils/mutil.h"
+#include "mailutils/util.h"
 #include "mailutils/stream.h"
 #include "mailutils/filter.h"
 #include "mailutils/md5.h"
 #include "mailutils/sha1.h"
 #include "mailutils/ldap.h"
-#include "mailutils/vartab.h"
 
 #include <ldap.h>
 #include <lber.h>
@@ -58,32 +60,98 @@ const char *default_field_map =
 "shell=loginShell";
 
 static struct mu_ldap_module_config ldap_param;
-
-int
-mu_ldap_module_init (enum mu_gocs_op op, void *data)
+
+static int
+cb_field_map (void *data, mu_config_value_t *val)
 {
-  struct mu_ldap_module_config *cfg = data;
+  char *err_term;
+  int rc = mu_cfg_field_map (val, &ldap_param.field_map, &err_term);
 
-  if (op != mu_gocs_op_set)
-    return 0;
-  
-  if (cfg)
-    ldap_param = *cfg;
+  if (rc)
+    {
+      if (err_term)
+	mu_error (_("error near %s: %s"), err_term, mu_strerror (rc));
+      else
+	mu_error ("%s", mu_strerror (rc));
+    }
 
+  return rc;
+}
+
+static struct mu_cfg_param mu_ldap_param[] = {
+  { "enable", mu_c_bool, &ldap_param.enable, 0, NULL,
+    N_("Enable LDAP lookups.") },
+  { "url", mu_c_string, &ldap_param.url, 0, NULL,
+    N_("Set URL of the LDAP server."),
+    N_("url") },
+  { "base", mu_c_string, &ldap_param.base, 0, NULL,
+    N_("Base DN for LDAP lookups."),
+    N_("dn") },
+  { "binddn", mu_c_string, &ldap_param.binddn, 0, NULL,
+    N_("DN for accessing LDAP database."),
+    N_("dn") },
+  { "passwd", mu_c_string, &ldap_param.passwd, 0, NULL,
+    N_("Password for use with binddn.") },
+  { "tls", mu_c_bool, &ldap_param.tls, 0, NULL,
+    N_("Use TLS encryption.") },
+  { "debug", mu_c_int, &ldap_param.debug, 0, NULL,
+    N_("Set LDAP debugging level.") },
+  { "field-map", mu_cfg_callback, NULL, 0, cb_field_map,
+    N_("Set a field-map for parsing LDAP replies.  The map is a "
+       "column-separated list of definitions.  Each definition has the "
+       "following form:\n"
+       "   <name: string>=<attr: string>\n"
+       "where <name> is one of the following: name, passwd, uid, gid, "
+       "gecos, dir, shell, mailbox, quota, and <attr> is the name of "
+       "the corresponding LDAP attribute."),
+    N_("map: definition") },
+  { "getpwnam", mu_c_string, &ldap_param.getpwnam_filter, 0, NULL,
+    N_("LDAP filter to use for getpwnam requests."),
+    N_("filter") },
+  { "getpwuid", mu_c_string, &ldap_param.getpwuid_filter, 0, NULL,
+    N_("LDAP filter to use for getpwuid requests."),
+    N_("filter") },
+  { NULL }
+};
+
+int									      
+mu_ldap_section_parser
+   (enum mu_cfg_section_stage stage, const mu_cfg_node_t *node,	      
+    const char *section_label, void **section_data,
+    void *call_data, mu_cfg_tree_t *tree)
+{									      
+  switch (stage)							      
+    {									      
+    case mu_cfg_section_start:
+      ldap_param.enable = 1;
+      break;
+    default:
+      break;
+    }									      
+  return 0;								      
+}
+
+
+static void
+module_init (void *ptr)
+{
   if (ldap_param.enable)
     {
       if (!ldap_param.getpwnam_filter)
-	ldap_param.getpwnam_filter = "(&(objectClass=posixAccount) (uid=%u))";
+	ldap_param.getpwnam_filter =
+	  "(&(objectClass=posixAccount) (uid=$user))";
       if (!ldap_param.getpwuid_filter)
 	ldap_param.getpwuid_filter =
-	  "&(objectClass=posixAccount) (uidNumber=%u))";
+	  "(&(objectClass=posixAccount) (uidNumber=$user))";
       if (!ldap_param.field_map)
 	{
-	  int d;
-	  mutil_parse_field_map (default_field_map, &ldap_param.field_map, &d);
+	  struct mu_config_value val;
+	  val.type = MU_CFG_STRING;
+	  val.v.string = default_field_map;
+	  if (mu_cfg_field_map (&val, &ldap_param.field_map, NULL))
+	    abort ();
 	}
     }
-  return 0;
 }
 
 static int
@@ -129,10 +197,9 @@ _mu_conn_setup (LDAP **pld)
 	    {
 	      /* if no host but a DN is provided, try DNS SRV to gather the
 		 host list */
-	      char	*domain = NULL, *hostlist = NULL, **hosts = NULL;
-	      size_t hostcnt;
-	      int i;
-	      int len_proto = strlen(lud->lud_scheme);
+	      char *domain = NULL, *hostlist = NULL;
+	      size_t i;
+	      struct mu_wordsplit ws;
 	      
 	      if (ldap_dn2domain (lud->lud_dn, &domain) || !domain)
 		{
@@ -148,16 +215,15 @@ _mu_conn_setup (LDAP **pld)
 			    domain);
 		  goto dnssrv_free;
 		}
-	      
-	      rc = mu_argcv_get (hostlist, " ", NULL, &hostcnt, &hosts);
-	      if (rc)
+
+	      if (mu_wordsplit (hostlist, &ws, MU_WRDSF_DEFFLAGS))
 		{
 		  mu_error (_("DNS SRV: could not parse hostlist=\"%s\": %s"),
-			    hostlist, mu_strerror (rc));
+			    hostlist, mu_wordsplit_strerror (&ws));
 		  goto dnssrv_free;
 		}
 	      
-	      tmp = realloc (urls, sizeof(char *) * (nurls + hostcnt + 1));
+	      tmp = realloc (urls, sizeof(char *) * (nurls + ws.ws_wordc + 1));
 	      if (!tmp)
 		{
 		  mu_error ("DNS SRV %s", mu_strerror (errno));
@@ -167,28 +233,23 @@ _mu_conn_setup (LDAP **pld)
 	      urls = tmp;
 	      urls[nurls] = NULL;
 	      
-	      for (i = 0; i < hostcnt; i++)
+	      for (i = 0; i < ws.ws_wordc; i++)
 		{
-		  size_t len = len_proto + sizeof "://" - 1
-		               + strlen (hosts[i])
-			       + 1;
-
 		  urls[nurls + i + 1] = NULL;
-		  urls[nurls + i] = malloc (len);
-		  if (!urls[nurls + i])
+		  rc = mu_asprintf (&urls[nurls + i],
+				    "%s://%s",
+				    lud->lud_scheme, ws.ws_wordv[i]);
+		  if (rc)
 		    {
-		      mu_error ("DNS SRV %s", mu_strerror (errno));
+		      mu_error ("DNS SRV %s", mu_strerror (rc));
 		      goto dnssrv_free;
 		    }
-		  
-		  snprintf (urls[nurls + i], len, "%s://%s",
-			    lud->lud_scheme, hosts[i]);
 		}
 	      
 	      nurls += i;
 	      
 	    dnssrv_free:
-	      mu_argcv_free (hostcnt, hosts);
+	      mu_wordsplit_free (&ws);
 	      ber_memfree (hostlist);
 	      ber_memfree (domain);
 	    }
@@ -250,24 +311,32 @@ _mu_conn_setup (LDAP **pld)
     }
   free (ldapuri);
   
+  ldap_set_option (ld, LDAP_OPT_PROTOCOL_VERSION, &protocol);
+
   if (ldap_param.tls)
     {
       rc = ldap_start_tls_s (ld, NULL, NULL);
       if (rc != LDAP_SUCCESS)
 	{
+	  char *msg = NULL;
+	  ldap_get_option (ld,
+			   LDAP_OPT_DIAGNOSTIC_MESSAGE,
+			   (void*)&msg);
+	  
 	  mu_error (_("ldap_start_tls failed: %s"), ldap_err2string (rc));
+	  mu_error (_("TLS diagnostics: %s"), msg);
+	  ldap_memfree (msg);
+
+	  ldap_unbind_ext (ld, NULL, NULL);
+	  
 	  return 1;
 	}
     }
 
-  ldap_set_option (ld, LDAP_OPT_PROTOCOL_VERSION, &protocol);
-
   /* FIXME: Timeouts, SASL, etc. */
   *pld = ld;
   return 0;
 }
-  
-     
 
 static int
 _mu_ldap_bind (LDAP *ld)
@@ -318,9 +387,8 @@ _mu_ldap_bind (LDAP *ld)
       || (info && info[0])
       || refs)
     {
-      /* FIXME: Use mu_debug_t for that */
-      mu_error ("ldap_bind: %s (%d)%s",
-		ldap_err2string (err), err, msgbuf ? msgbuf : "");
+      /* FIXME: Use debug output for that */
+      mu_error ("ldap_bind: %s (%d)%s", ldap_err2string (err), err, msgbuf);
 
       if (matched && *matched) 
 	mu_error ("matched DN: %s", matched);
@@ -376,7 +444,11 @@ _construct_attr_array (size_t *pargc, char ***pargv)
     {
       char **str;
       mu_iterator_current (itr, (void**) &str); 
-      argv[i] = strdup (*str);
+      if ((argv[i] = strdup (*str)) == NULL)
+        {
+          mu_argcv_free (i, argv);
+          return ENOMEM;
+        }
     }
   mu_iterator_destroy (&itr);
   argv[i] = NULL;
@@ -424,6 +496,33 @@ _free_partial_auth_data (struct mu_auth_data *d)
   free (d->mailbox);
 }
 
+static int
+_assign_partial_auth_data (struct mu_auth_data *d, const char *key,
+			   const char *val)
+{
+  int rc = 0;
+  
+  if (strcmp (key, MU_AUTH_NAME) == 0)
+    rc = (d->name = strdup (val)) ? 0 : errno;
+  else if (strcmp (key, MU_AUTH_PASSWD) == 0)
+    rc = (d->passwd = strdup (val)) ? 0 : errno;
+  else if (strcmp (key, MU_AUTH_UID) == 0)
+    d->uid = atoi (val);
+  else if (strcmp (key, MU_AUTH_GID) == 0)
+    d->gid = atoi (val);
+  else if (strcmp (key, MU_AUTH_GECOS) == 0)
+    rc = (d->gecos = strdup (val)) ? 0 : errno;
+  else if (strcmp (key, MU_AUTH_DIR) == 0)
+    rc = (d->dir = strdup (val)) ? 0 : errno;
+  else if (strcmp (key, MU_AUTH_SHELL) == 0)   
+    rc = (d->shell = strdup (val)) ? 0 : errno;
+  else if (strcmp (key, MU_AUTH_MAILBOX) == 0)
+    rc = (d->mailbox = strdup (val)) ? 0 : errno;
+  else if (strcmp (key, MU_AUTH_QUOTA) == 0)   
+    get_quota (&d->quota, val);
+  return rc;
+}
+
 static int
 _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
 			struct mu_auth_data **return_data)
@@ -437,10 +536,8 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
   
   memset (&d, 0, sizeof d);
   
-  /* FIXME: should use mu_debug_t */
   rc = ldap_get_dn_ber (ld, msg, &ber, &bv);
   ufn = ldap_dn2ufn (bv.bv_val);
-  mu_error ("INFO: %s", ufn);
   ldap_memfree (ufn);
   
   mu_assoc_get_iterator (ldap_param.field_map, &itr);
@@ -463,26 +560,14 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
 	  return MU_ERR_READ;
 	}
       
-      if (strcmp (key, MU_AUTH_NAME) == 0)
-	d.name = strdup (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_PASSWD) == 0)
-	d.passwd = strdup (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_UID) == 0)
-	d.uid = atoi (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_GID) == 0)
-	d.gid = atoi (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_GECOS) == 0)
-	d.gecos = strdup (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_DIR) == 0)
-	d.dir = strdup (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_SHELL) == 0)   
-	d.shell = strdup (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_MAILBOX) == 0)
-	d.mailbox = strdup (values[0]->bv_val);
-      else if (strcmp (key, MU_AUTH_QUOTA) == 0)   
-	get_quota (&d.quota, values[0]->bv_val);
+      rc = _assign_partial_auth_data (&d, key, values[0]->bv_val);
       
       ldap_value_free_len (values);
+      if (rc)
+	{
+	  _free_partial_auth_data (&d);
+	  return rc;
+	}
     }
   
   rc = mu_auth_data_alloc (return_data,
@@ -509,33 +594,43 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
 		 struct mu_auth_data **return_data)
 {
   int rc;
-  char *filter;
   char **attrs;
   size_t nattrs;
   LDAPMessage *res, *msg;
   ber_int_t msgid;
-  mu_vartab_t vtab;
+  const char *env[3];
+  struct mu_wordsplit ws;
 
   rc = _construct_attr_array (&nattrs, &attrs);
   if (rc)
     return rc;
 
-  mu_vartab_create (&vtab);
-  mu_vartab_define (vtab, "user", key, 1);
-  mu_vartab_define (vtab, "u", key, 1);
-  rc = mu_vartab_expand (vtab, filter_pat, &filter);
-  mu_vartab_destroy (&vtab);
-  if (rc)
-    {
-      mu_argcv_free (nattrs, attrs);
-      return ENOMEM;
-    }
+  env[0] = "user";
+  env[1] = key;
+  env[2] = NULL;
 
+  ws.ws_env = env;
+  if (mu_wordsplit (filter_pat, &ws,
+		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
+		    MU_WRDSF_ENV | MU_WRDSF_ENV_KV))
+    {
+      mu_error (_("cannot expand line `%s': %s"), filter_pat,
+		mu_wordsplit_strerror (&ws));
+      return MU_ERR_FAILURE;
+    }
+  else if (ws.ws_wordc == 0)
+    {
+      mu_error (_("expanding %s yields empty string"), filter_pat);
+      mu_wordsplit_free (&ws);
+      mu_argcv_free (nattrs, attrs);
+      return MU_ERR_FAILURE;
+    }
+  
   rc = ldap_search_ext (ld, ldap_param.base, LDAP_SCOPE_SUBTREE,
-			filter, attrs, 0,
+			ws.ws_wordv[0], attrs, 0,
 			NULL, NULL, NULL, -1, &msgid);
+  mu_wordsplit_free (&ws);
   mu_argcv_free (nattrs, attrs);
-  free (filter);
 
   if (rc != LDAP_SUCCESS)
     {
@@ -594,15 +689,12 @@ chk_md5 (const char *db_pass, const char *pass)
   mu_md5_process_bytes (pass, strlen (pass), &md5context);
   mu_md5_finish_ctx (&md5context, md5digest);
 
-  mu_memory_stream_create (&str, NULL, MU_STREAM_NO_CHECK);
-  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE,
-		    MU_STREAM_READ | MU_STREAM_NO_CHECK);
-  mu_stream_open (str);
-  mu_stream_sequential_write (str, db_pass, strlen (db_pass));
+  mu_static_memory_stream_create (&str, db_pass, strlen (db_pass));
+  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
+  mu_stream_unref (str);
 
-  mu_stream_read (flt, d1, sizeof d1, 0, NULL);
-  mu_stream_destroy (&flt, NULL);
-  mu_stream_destroy (&str, NULL);
+  mu_stream_read (flt, (char*) d1, sizeof d1, NULL);
+  mu_stream_destroy (&flt);
   
   return memcmp (md5digest, d1, sizeof md5digest) == 0 ?
                   0 : MU_ERR_AUTH_FAILURE;
@@ -618,24 +710,20 @@ chk_smd5 (const char *db_pass, const char *pass)
   mu_stream_t str = NULL, flt = NULL;
   size_t size;
   
-  mu_memory_stream_create (&str, NULL, MU_STREAM_NO_CHECK);
-  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE,
-		    MU_STREAM_READ | MU_STREAM_NO_CHECK);
-  mu_stream_open (str);
   size = strlen (db_pass);
-  mu_stream_sequential_write (str, db_pass, size);
+  mu_static_memory_stream_create (&str, db_pass, size);
+  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
+  mu_stream_unref (str);
 
   d1 = malloc (size);
   if (!d1)
     {
-      mu_stream_destroy (&flt, NULL);
-      mu_stream_destroy (&str, NULL);
+      mu_stream_destroy (&flt);
       return ENOMEM;
     }
   
-  mu_stream_read (flt, d1, size, 0, &size);
-  mu_stream_destroy (&flt, NULL);
-  mu_stream_destroy (&str, NULL);
+  mu_stream_read (flt, (char*) d1, size, &size);
+  mu_stream_destroy (&flt);
 
   if (size <= 16)
     {
@@ -666,15 +754,12 @@ chk_sha (const char *db_pass, const char *pass)
   mu_sha1_process_bytes (pass, strlen (pass), &sha1context);
   mu_sha1_finish_ctx (&sha1context, sha1digest);
 
-  mu_memory_stream_create (&str, NULL, MU_STREAM_NO_CHECK);
-  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE,
-		    MU_STREAM_READ | MU_STREAM_NO_CHECK);
-  mu_stream_open (str);
-  mu_stream_sequential_write (str, db_pass, strlen (db_pass));
+  mu_static_memory_stream_create (&str, db_pass, strlen (db_pass));
+  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
+  mu_stream_unref (str);
 
-  mu_stream_read (flt, d1, sizeof d1, 0, NULL);
-  mu_stream_destroy (&flt, NULL);
-  mu_stream_destroy (&str, NULL);
+  mu_stream_read (flt, (char*) d1, sizeof d1, NULL);
+  mu_stream_destroy (&flt);
   
   return memcmp (sha1digest, d1, sizeof sha1digest) == 0 ?
                   0 : MU_ERR_AUTH_FAILURE;
@@ -690,24 +775,20 @@ chk_ssha (const char *db_pass, const char *pass)
   mu_stream_t str = NULL, flt = NULL;
   size_t size;
   
-  mu_memory_stream_create (&str, NULL, MU_STREAM_NO_CHECK);
-  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE,
-		    MU_STREAM_READ | MU_STREAM_NO_CHECK);
-  mu_stream_open (str);
   size = strlen (db_pass);
-  mu_stream_sequential_write (str, db_pass, size);
+  mu_static_memory_stream_create (&str, db_pass, size);
+  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
+  mu_stream_unref (str);
 
   d1 = malloc (size);
   if (!d1)
     {
-      mu_stream_destroy (&flt, NULL);
-      mu_stream_destroy (&str, NULL);
+      mu_stream_destroy (&flt);
       return ENOMEM;
     }
   
-  mu_stream_read (flt, d1, size, 0, &size);
-  mu_stream_destroy (&flt, NULL);
-  mu_stream_destroy (&str, NULL);
+  mu_stream_read (flt, (char*) d1, size, &size);
+  mu_stream_destroy (&flt);
 
   if (size <= 16)
     {
@@ -839,19 +920,21 @@ mu_auth_ldap_user_by_uid (struct mu_auth_data **return_data,
 
 
 #else
-# define mu_ldap_module_init NULL
+# define module_init NULL
 # define mu_ldap_authenticate mu_auth_nosupport
 # define mu_auth_ldap_user_by_name mu_auth_nosupport 
 # define mu_auth_ldap_user_by_uid mu_auth_nosupport
+# define mu_ldap_param NULL
+
 #endif
 
 struct mu_auth_module mu_auth_ldap_module = {
-  "ldap",
-  mu_ldap_module_init,
-  mu_ldap_authenticate,
-  NULL,
-  mu_auth_ldap_user_by_name,
-  NULL,
-  mu_auth_ldap_user_by_uid,
-  NULL
+  .name = "ldap",
+  .commit = module_init,
+  .handler = {
+    [mu_auth_authenticate] = mu_ldap_authenticate,
+    [mu_auth_getpwnam]     = mu_auth_ldap_user_by_name,
+    [mu_auth_getpwuid]     = mu_auth_ldap_user_by_uid
+  },
+  .cfg = mu_ldap_param
 };
