@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005, 2007, 2009-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2003, 2005, 2007, 2009-2012, 2014-2016 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -183,7 +183,7 @@ mime_descend (struct mime_descend_closure *closure,
   mu_message_get_header (closure->message, &hdr);
   util_get_hdr_value (hdr, MU_HEADER_CONTENT_TYPE, &type);
   if (type == NULL)
-    type = xstrdup ("text/plain");
+    type = mu_strdup ("text/plain");
   get_content_encoding (hdr, &encoding);
 
   closure->type = type;
@@ -346,7 +346,7 @@ get_content_encoding (mu_header_t hdr, char **value)
     {
       if (encoding)
 	free (encoding);
-      encoding = xstrdup ("7bit"); /* Default.  */
+      encoding = mu_strdup ("7bit"); /* Default.  */
     }
   *value = encoding;
   return 0;
@@ -411,7 +411,7 @@ run_metamail (const char *mailcap_cmd, mu_message_t mesg)
 	  status = mu_message_get_streamref (mesg, &stream);
 	  if (status)
 	    {
-	      mu_error ("mu_message_get_stream: %s", mu_strerror (status));
+	      mu_error ("mu_message_get_streamref: %s", mu_strerror (status));
 	      break;
 	    }
 
