@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2002, 2005, 2007, 2010-2012, 2014-2016 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -29,8 +28,8 @@ mail_summary (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
   mu_attribute_t attr;
   size_t msgno;
   size_t count = 0;
-  int mseen = 0, mnew = 0, mdelete = 0;
-  int first_new = 0, first_unread = 0;
+  unsigned long mseen = 0, mnew = 0, mdelete = 0;
+  size_t first_new = 0, first_unread = 0;
 
   mu_mailbox_messages_count (mbox, &count);
   for (msgno = 1; msgno <= count; msgno++)
@@ -61,16 +60,18 @@ mail_summary (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
   {
     mu_url_t url = NULL;
     mu_mailbox_get_url (mbox, &url);
-    printf("\"%s\": ", mu_url_to_string (url));
+    mu_printf ("\"%s\": ", util_url_to_string (url));
   }
-  printf (ngettext ("%d message", "%d messages", count), count);
+  mu_printf (ngettext ("%lu message", "%lu messages",
+		       (unsigned long) count), (unsigned long) count);
   if (mnew > 0)
-    printf (ngettext (" %d new", " %d new", mnew), mnew);
+    mu_printf (ngettext (" %lu new", " %lu new", mnew), mnew);
   if (mseen > 0)
-    printf (ngettext (" %d unread", " %d unread", mseen), mseen);
+    mu_printf (ngettext (" %lu unread", " %lu unread", mseen), mseen);
   if (mdelete > 0)
-    printf (ngettext (" %d deleted", " %d deleted", mdelete), mdelete);
-  printf("\n");
+    mu_printf (ngettext (" %lu deleted", " %lu deleted", mdelete),
+	       mdelete);
+  mu_printf ("\n");
 
   /* Set the cursor.  */
   set_cursor ((first_new == 0) ? ((first_unread == 0) ?
