@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2004, 2005, 2006, 2007, 
-   2008 Free Software Foundation, Inc.
+   Copyright (C) 2003-2012, 2014-2016 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH ali command */
 
@@ -26,81 +23,41 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
-const char *program_version = "ali (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU MH ali")"\v"
-N_("Options marked with `*' are not yet implemented.\n"
-"Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("aliases ...");
-
-/* GNU options */
-static struct argp_option options[] = {
-  {"alias",  ARG_ALIAS, N_("FILE"), 0,
-   N_("Use the additional alias FILE")},
-  {"noalias", ARG_NOALIAS, NULL, 0,
-   N_("Do not read the system alias file") },
-  {"list", ARG_LIST, N_("BOOL"),  OPTION_ARG_OPTIONAL,
-   N_("List each address on a separate line") },
-  {"normalize", ARG_NORMALIZE, N_("BOOL"),  OPTION_ARG_OPTIONAL,
-   N_("Try to determine the official hostname for each address") },
-  {"user", ARG_USER, N_("BOOL"),  OPTION_ARG_OPTIONAL,
-   N_("List the aliases that expand to given addresses") },
-  {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
-  { 0 }
-};
-
-/* Traditional MH options */
-struct mh_option mh_option[] = {
-  { "alias", 1, MH_OPT_ARG, "aliasfile" },
-  { "noalias", 3, },
-  { "list", 1, MH_OPT_BOOL },
-  { "normalize", 3, MH_OPT_BOOL },
-  { "user", 1, MH_OPT_BOOL },
-  { 0 }
-};
+static char prog_doc[] = N_("List mail aliases");
+static char args_doc[] = N_("ALIAS [ALIAS...]");
 
 static int list_mode;
 static int user_mode;
 static int normalize_mode;
 static int nolist_mode;
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static void
+alias_handler (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
-  switch (key)
-    {
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
-    case ARG_ALIAS:
-      mh_alias_read (arg, 1);
-      break;
-
-    case ARG_NOALIAS:
-      nolist_mode = 1;
-      break;
-      
-    case ARG_LIST:
-      list_mode = is_true (arg);
-      break;
-	
-    case ARG_NORMALIZE:
-      normalize_mode = is_true (arg);
-      break;
-
-    case ARG_USER:
-      user_mode = is_true (arg);
-      break;
-
-    default:
-      return 1;
-    }
-  return 0;
+  mh_alias_read (arg, 1);
 }
 
+static struct mu_option options[] = {
+  { "alias",  0, N_("FILE"), MU_OPTION_DEFAULT,
+    N_("use the additional alias FILE"),
+    mu_c_string, NULL, alias_handler },
+  { "noalias", 0, NULL, 0,
+    N_("do not read the system alias file"),
+    mu_c_int, &nolist_mode, NULL, "1" },
+  { "list", 0, NULL,  MU_OPTION_DEFAULT,
+    N_("list each address on a separate line"),
+    mu_c_bool, &list_mode },
+  { "normalize", 0, NULL, MU_OPTION_DEFAULT,
+    N_("try to determine the official hostname for each address"),
+    mu_c_bool, &normalize_mode },
+  { "user", 0, NULL,  MU_OPTION_DEFAULT,
+    N_("list the aliases that expand to given addresses"),
+    mu_c_bool, &user_mode },
+  MU_OPTION_END
+};
+
 static int
-getcols ()
+getcols (void)
 {
   struct winsize ws;
 
@@ -127,39 +84,34 @@ ali_print_name_list (mu_list_t list, int off)
   
   if (list_mode)
     {
-      mu_iterator_first (itr);
-      mu_iterator_current (itr, (void **)&item);
-      printf ("%s\n", item);
-      for (mu_iterator_next (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
+      for (mu_iterator_first (itr);
+	   !mu_iterator_is_done (itr); mu_iterator_next (itr))
 	{
-	  int len;
 	  mu_iterator_current (itr, (void **)&item);
-	  len = off + strlen (item);
-	  printf ("%*.*s\n", len, len, item);
+	  printf ("%s\n", item);
 	}
     }
   else
     {
       int ncol = getcols ();
       int n = off;
+      int i = 0;
       
-      mu_iterator_first (itr);
-
-      for (;;)
+      for (mu_iterator_first (itr), i = 0;
+	   !mu_iterator_is_done (itr);
+	   mu_iterator_next (itr), i++)
 	{
 	  int len;
 
+	  if (i > 0)
+	    n += printf (", ");
+	  
 	  mu_iterator_current (itr, (void **)&item);
 	  len = strlen (item) + 2;
 	  if (n + len > ncol)
 	    n = printf ("\n ");
 
 	  len = printf ("%s", item);
-	  mu_iterator_next (itr);
-	  if (!mu_iterator_is_done (itr))
-	    len += printf (", ");
-	  else
-	    break;
 	  n += len;
 	}
       printf ("\n");
@@ -190,17 +142,7 @@ ali_print_name (char *name)
 int
 main (int argc, char **argv)
 {
-  int index;
-  
-  /* Native Language Support */
-  MU_APP_INIT_NLS ();
-
-  mh_argp_init (program_version);
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
-		 opt_handler, NULL, &index);
-
-  argc -= index;
-  argv += index;
+  mh_getopt (&argc, &argv, options, 0, args_doc, prog_doc, NULL);
 
   if (!nolist_mode)
     mh_read_aliases ();
