@@ -1,6 +1,5 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2009-2012, 2014-2016 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,10 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include "libmu_py.h"
 
@@ -97,18 +94,62 @@ api_sieve_machine_init (PyObject *self, PyObject *args)
 {
   int status;
   PySieveMachine *py_mach;
-  struct _mu_py_sieve_logger *s;
-
+  mu_stream_t str, estr;
+  
   if (!PyArg_ParseTuple (args, "O!", &PySieveMachineType, &py_mach))
     return NULL;
 
-  if (!(s = calloc (1, sizeof (*s))))
-    return NULL;
-
-  status = mu_sieve_machine_init (&py_mach->mach, s);
+  status = mu_memory_stream_create (&str, MU_STREAM_RDWR);
+  if (status)
+    return _ro (PyInt_FromLong (status));
+  status = mu_log_stream_create (&estr, str);
+  mu_stream_unref (str);
+  if (status)
+    return _ro (PyInt_FromLong (status));
+  
+  status = mu_sieve_machine_create (&py_mach->mach);
+  if (status == 0)
+    mu_sieve_set_diag_stream (py_mach->mach, estr);
+  mu_stream_unref (estr);
   return _ro (PyInt_FromLong (status));
 }
 
+static PyObject *
+api_sieve_machine_error_text (PyObject *self, PyObject *args)
+{
+  int status;
+  PySieveMachine *py_mach;
+  mu_stream_t estr;
+  mu_transport_t trans[2];
+  PyObject *retval;
+  mu_off_t length = 0;
+  
+  if (!PyArg_ParseTuple (args, "O!", &PySieveMachineType, &py_mach))
+    return NULL;
+  if (!py_mach->mach)
+    PyErr_SetString (PyExc_RuntimeError, "Uninitialized Sieve machine");
+      
+  mu_sieve_get_diag_stream (py_mach->mach, &estr);
+  status = mu_stream_ioctl (estr, MU_IOCTL_TRANSPORT, MU_IOCTL_OP_GET,
+			    trans);
+  if (status == 0)
+    {
+      mu_stream_t str = (mu_stream_t) trans[0];
+
+      mu_stream_size (str, &length);
+      status = mu_stream_ioctl (str, MU_IOCTL_TRANSPORT, MU_IOCTL_OP_GET,
+				trans);
+      mu_stream_truncate (str, 0);
+    }
+  
+  if (status)
+    PyErr_SetString (PyExc_RuntimeError, mu_strerror (status));
+  retval = PyString_FromStringAndSize ((char*) trans[0], length);
+
+  mu_stream_unref (estr);
+  return _ro (retval);
+}
+
 static PyObject *
 api_sieve_machine_destroy (PyObject *self, PyObject *args)
 {
@@ -117,12 +158,13 @@ api_sieve_machine_destroy (PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple (args, "O!", &PySieveMachineType, &py_mach))
     return NULL;
 
-  if (py_mach->mach) {
-    struct _mu_py_sieve_logger *s = mu_sieve_get_data (py_mach->mach);
-    if (s)
-      free (s);
-    mu_sieve_machine_destroy (&py_mach->mach);
-  }
+  if (py_mach->mach)
+    {
+      struct _mu_py_sieve_logger *s = mu_sieve_get_data (py_mach->mach);
+      if (s)
+	free (s);
+      mu_sieve_machine_destroy (&py_mach->mach);
+    }
   return _ro (Py_None);
 }
 
@@ -193,223 +235,71 @@ api_sieve_message (PyObject *self, PyObject *args)
   return _ro (PyInt_FromLong (status));
 }
 
-static int
-_sieve_error_printer (void *data, const char *fmt, va_list ap)
-{
-  char *buf = NULL;
-  size_t buflen = 0;
-  PyObject *py_args = PyTuple_New (1);
-
-  if (py_args)
-    {
-      struct _mu_py_sieve_logger *s = data;
-      PyObject *py_fnc = s->py_error_printer;
-
-      if (mu_vasnprintf (&buf, &buflen, fmt, ap))
-	return 0;
-      PyTuple_SetItem (py_args, 0, PyString_FromString (buf ? buf : ""));
-      if (buf)
-	free (buf);
-
-      if (py_fnc && PyCallable_Check (py_fnc))
-	PyObject_CallObject (py_fnc, py_args);
-
-      Py_DECREF (py_args);
-    }
-  return 0;
-}
-
-static int
-_sieve_parse_error_printer (void *data, const char *filename, int lineno,
-			    const char *fmt, va_list ap)
-{
-  PyObject *py_args;
-  PyObject *py_dict;
-
-  py_dict = PyDict_New ();
-  if (py_dict)
-    {
-      char *buf = NULL;
-      size_t buflen = 0;
-
-      PyDict_SetItemString (py_dict, "filename",
-			    PyString_FromString (filename));
-      PyDict_SetItemString (py_dict, "lineno", PyInt_FromLong (lineno));
-
-      if (mu_vasnprintf (&buf, &buflen, fmt, ap))
-	return 0;
-      PyDict_SetItemString (py_dict, "text",
-			    PyString_FromString (buf ? buf : ""));
-      if (buf)
-	free (buf);
-
-      py_args = PyTuple_New (1);
-      if (py_args)
-	{
-	  struct _mu_py_sieve_logger *s = data;
-	  PyObject *py_fnc = s->py_parse_error_printer;
-
-	  Py_INCREF (py_dict);
-	  PyTuple_SetItem (py_args, 0, py_dict);
-
-	  if (py_fnc && PyCallable_Check (py_fnc))
-	    PyObject_CallObject (py_fnc, py_args);
-
-	  Py_DECREF (py_dict);
-	  Py_DECREF (py_args);
-	}
-    }
-  return 0;
-}
-
-static int
-_sieve_debug_printer (void *data, const char *fmt, va_list ap)
-{
-  char *buf = NULL;
-  size_t buflen = 0;
-  PyObject *py_args = PyTuple_New (1);
-
-  if (py_args)
-    {
-      struct _mu_py_sieve_logger *s = data;
-      PyObject *py_fnc = s->py_debug_printer;
-
-      if (mu_vasnprintf (&buf, &buflen, fmt, ap))
-	return 0;
-      PyTuple_SetItem (py_args, 0, PyString_FromString (buf ? buf : ""));
-      if (buf)
-	free (buf);
-
-      if (py_fnc && PyCallable_Check (py_fnc))
-	PyObject_CallObject (py_fnc, py_args);
-
-      Py_DECREF (py_args);
-    }
-  return 0;
-}
-
 static void
-_sieve_action_printer (void *data, const mu_sieve_locus_t *locus,
-		       size_t msgno, mu_message_t msg,
+_sieve_action_printer (mu_sieve_machine_t mach,
 		       const char *action, const char *fmt, va_list ap)
 {
   PyObject *py_args;
-  PyObject *py_dict;
+  PyObject *py_dict = PyDict_New ();
+  PyStream *py_stm;
 
-  py_dict = PyDict_New ();
-  if (py_dict)
+  if (!py_dict)
+    return;
+  py_stm = PyStream_NEW ();
+  if (py_stm)
     {
+      PyMessage *py_msg = PyMessage_NEW ();
       char *buf = NULL;
       size_t buflen = 0;
-      PyMessage *py_msg = PyMessage_NEW ();
 
-      py_msg->msg = msg;
-      Py_INCREF (py_msg);
-
-      PyDict_SetItemString (py_dict, "source_file",
-			    PyString_FromString (locus->source_file));
-      PyDict_SetItemString (py_dict, "source_line",
-			    PyInt_FromLong (locus->source_line));
-      PyDict_SetItemString (py_dict, "msgno",
-			    PyInt_FromLong (msgno));
-      PyDict_SetItemString (py_dict, "msg", (PyObject *)py_msg);
-      PyDict_SetItemString (py_dict, "action",
-			    PyString_FromString (action ? action : ""));
-
-      if (mu_vasnprintf (&buf, &buflen, fmt, ap))
-	return;
-      PyDict_SetItemString (py_dict, "text",
-			    PyString_FromString (buf ? buf : ""));
-      if (buf)
-	free (buf);
-
-      py_args = PyTuple_New (1);
-      if (py_args)
+      if (py_msg)
 	{
-	  struct _mu_py_sieve_logger *s = data;
-	  PyObject *py_fnc = s->py_action_printer;
+	  PyStream *py_stm = PyStream_NEW ();
+	  if (py_stm)
+	    {
+	      size_t msgno;
+	      
+	      mu_sieve_get_diag_stream (mach, &py_stm->stm);
+	      msgno = mu_sieve_get_message_num (mach);
+	      
+	      py_msg->msg = mu_sieve_get_message (mach);
+	      Py_INCREF (py_msg);
 
-	  Py_INCREF (py_dict);
-	  PyTuple_SetItem (py_args, 0, py_dict);
+	      PyDict_SetItemString (py_dict, "msgno",
+				    PyInt_FromSize_t (msgno));
+	      PyDict_SetItemString (py_dict, "msg", (PyObject *)py_msg);
+	      PyDict_SetItemString (py_dict, "action",
+				    PyString_FromString (action ? action : ""));
 
-	  if (py_fnc && PyCallable_Check (py_fnc))
-	    PyObject_CallObject (py_fnc, py_args);
+	      if (mu_vasnprintf (&buf, &buflen, fmt, ap))
+		{
+		  mu_stream_destroy (&py_stm->stm);
+		  return;
+		}
+	      PyDict_SetItemString (py_dict, "text",
+				    PyString_FromString (buf ? buf : ""));
+	      free (buf);
 
-	  Py_DECREF (py_dict);
-	  Py_DECREF (py_args);
+	      py_args = PyTuple_New (1);
+	      if (py_args)
+		{
+		  struct _mu_py_sieve_logger *s = mu_sieve_get_data (mach);
+		  PyObject *py_fnc = s->py_action_printer;
+
+		  Py_INCREF (py_dict);
+		  PyTuple_SetItem (py_args, 0, py_dict);
+		  
+		  if (py_fnc && PyCallable_Check (py_fnc))
+		    PyObject_CallObject (py_fnc, py_args);
+
+		  Py_DECREF (py_dict);
+		  Py_DECREF (py_args);
+		}
+	    }
 	}
     }
 }
 
-static PyObject *
-api_sieve_set_error (PyObject *self, PyObject *args)
-{
-  PySieveMachine *py_mach;
-  PyObject *py_fnc;
-
-  if (!PyArg_ParseTuple (args, "O!O", &PySieveMachineType, &py_mach, &py_fnc))
-    return NULL;
-
-  if (py_fnc && PyCallable_Check (py_fnc)) {
-    mu_sieve_machine_t mach = py_mach->mach;
-    struct _mu_py_sieve_logger *s = mu_sieve_get_data (mach);
-    s->py_error_printer = py_fnc;
-    Py_INCREF (py_fnc);
-    mu_sieve_set_error (py_mach->mach, _sieve_error_printer);
-  }
-  else {
-      PyErr_SetString (PyExc_TypeError, "");
-      return NULL;
-  }
-  return _ro (Py_None);
-}
-
-static PyObject *
-api_sieve_set_parse_error (PyObject *self, PyObject *args)
-{
-  PySieveMachine *py_mach;
-  PyObject *py_fnc;
-
-  if (!PyArg_ParseTuple (args, "O!O", &PySieveMachineType, &py_mach, &py_fnc))
-    return NULL;
-
-  if (py_fnc && PyCallable_Check (py_fnc)) {
-    mu_sieve_machine_t mach = py_mach->mach;
-    struct _mu_py_sieve_logger *s = mu_sieve_get_data (mach);
-    s->py_parse_error_printer = py_fnc;
-    Py_INCREF (py_fnc);
-    mu_sieve_set_parse_error (py_mach->mach, _sieve_parse_error_printer);
-  }
-  else {
-      PyErr_SetString (PyExc_TypeError, "");
-      return NULL;
-  }
-  return _ro (Py_None);
-}
-
-static PyObject *
-api_sieve_set_debug (PyObject *self, PyObject *args)
-{
-  PySieveMachine *py_mach;
-  PyObject *py_fnc;
-
-  if (!PyArg_ParseTuple (args, "O!O", &PySieveMachineType, &py_mach, &py_fnc))
-    return NULL;
-
-  if (py_fnc && PyCallable_Check (py_fnc)) {
-    mu_sieve_machine_t mach = py_mach->mach;
-    struct _mu_py_sieve_logger *s = mu_sieve_get_data (mach);
-    s->py_debug_printer = py_fnc;
-    Py_INCREF (py_fnc);
-    mu_sieve_set_debug (py_mach->mach, _sieve_debug_printer);
-  }
-  else {
-      PyErr_SetString (PyExc_TypeError, "");
-      return NULL;
-  }
-  return _ro (Py_None);
-}
-
 static PyObject *
 api_sieve_set_logger (PyObject *self, PyObject *args)
 {
@@ -440,6 +330,9 @@ static PyMethodDef methods[] = {
   { "machine_destroy", (PyCFunction) api_sieve_machine_destroy, METH_VARARGS,
     "Destroy Sieve 'machine'." },
 
+  { "errortext", (PyCFunction) api_sieve_machine_error_text, METH_VARARGS,
+    "Return the most recent error description." },
+  
   { "compile", (PyCFunction) api_sieve_compile, METH_VARARGS,
     "Compile the sieve script from the file 'name'." },
 
@@ -454,15 +347,6 @@ static PyMethodDef methods[] = {
     "Execute the code from the given instance of sieve machine "
     "'mach' over the 'message'. " },
 
-  { "set_debug", (PyCFunction) api_sieve_set_debug, METH_VARARGS,
-    "" },
-
-  { "set_error", (PyCFunction) api_sieve_set_error, METH_VARARGS,
-    "" },
-
-  { "set_parse_error", (PyCFunction) api_sieve_set_parse_error, METH_VARARGS,
-    "" },
-
   { "set_logger", (PyCFunction) api_sieve_set_logger, METH_VARARGS,
     "" },
 
