@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2010-2012, 2014-2016 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -189,7 +189,7 @@ static int
 odbc_release_result (mu_sql_connection_t conn)
 {
   struct mu_odbc_data *dp = conn->data;
-  mu_list_do (dp->result, odbc_free_char_data, NULL);
+  mu_list_foreach (dp->result, odbc_free_char_data, NULL);
   mu_list_destroy (&dp->result);
   mu_argcv_free (dp->fcount, dp->fnames);
   dp->fcount = 0;
@@ -252,9 +252,9 @@ odbc_get_column (mu_sql_connection_t conn,
       return MU_ERR_SQL;
     }
   
-  *pdata = strdup (buffer);
-  mu_list_append (dp->result, *pdata);
-  return 0;
+  if ((*pdata = strdup (buffer)) == NULL)
+    return ENOMEM;
+  return mu_list_append (dp->result, *pdata);
 }
 
 /* FIXME: untested */
